function [clusters, clusterIndx, clStd] = cluster_through_gmd(pnts, display, gmdata)
%% clusters = cluster_through_gmd(pnts)
%  
% This function fits an increasing number of gaussian mixture distribution
% to the 3D point data until the log-likelihood falls below 0.8 or starts
% to increase again

clStd = false;
center = mean(pnts,2);
thresh = 2/100 * center(3);
%thresh = 30;

if(nargin < 3)
    gmdata = pnts;
end

if(nargin < 2)
    display = false;
end

clusters = {pnts};
clusterIndx = {[1:size(pnts,2)]};

%% The pnts are nearly arranged as a plane 

[coeff, score] = princomp(pnts(1:3,:)');
stdtest = mean(abs(score(:,3)));

if(stdtest < thresh) 
    clStd = true;
    
    if(display)
        show_clusters(clusters);
    end
    
    return; 
end

%% 

obj = gmdistribution.fit(gmdata', 1);
log1 = obj.NlogL;

cl_set = {clusters};
cl_indx = {clusterIndx};

log = 1;
%grad = 0;

slog = 100*log;
sgrad = [];

i = 2;
while(1)
    
    %if(log < 0.8 | (i > 5 & grad < 0)) break; end
    
    try
        obj = gmdistribution.fit(gmdata', i);
    catch
        i = 2;
        log = 1;
        %grad = 0;
        %sgrad = [];
        slog = 100*log;

        continue;
    end
    
    idx = cluster(obj, gmdata');
    [cl, clIndx] = pnts2clusters(pnts, idx);
    
    cl_set = [cl_set {cl}];
    cl_indx = [cl_indx {clIndx}];
    
    log = obj.NlogL / log1;
    %log = obj.NlogL;
    slog = [slog 100*log];
    %plot(slog); drawnow; axis equal;
    
    if(i == 2)
        grad = slog(i) - slog(i-1);
    else
        grad = (slog(i) - slog(i-2)) / 2;
    end
    
    %sgrad = [sgrad grad];
    
    
    if(grad > -1)
        clusters = cl_set{i-1};
        clusterIndx = cl_indx{i-1};
        break;
    end
    
    if(~obj.Converged)
        clusters = cl_set{i-1};
        clusterIndx = cl_indx{i-1};
        break;
    end
    
    i = i+1;
    
end
    
if(display)
    show_clusters(clusters);
end