function [clusters, clusterIndx] = cluster_through_histNormals(fr, cam, display)
%% [clusters, clusterIndx] = cluster_through_histNormals(fr, cam, display)

%% some defaults:
if(nargin < 3)
    display = false;
end

if(~isfield(fr, 'normals'))
    seq.frames = fr;
    seq.camType = cam;
    
    seq = preprocessing(seq, 'normals');
    fr = seq.frames;
end

deviation = fr.dev;
thres_dev = 1 * mean(deviation) + std(deviation);
planarIndx = find(deviation <= thres_dev);
fr.usage = intersect(fr.usage, planarIndx);

%plot_3Dpoints(fr.points3D(:, fr.usage)); drawnow;

%% compute angles along the three axis X, Y, Z:

X = fr.normals(1, fr.usage);
Y = fr.normals(2, fr.usage);
Z = fr.normals(3, fr.usage);
XYZ = sqrt(X.^2 + Y.^2 + Z.^2);

angle_x = abs(asin(X./XYZ));
angle_y = abs(asin(Y./XYZ));
angle_z = abs(asin(Z./XYZ));

%angle = max([angle_x; angle_y; angle_z]);

%% compute histograms over these angles:
edges = [0:pi/4.5:(pi/2-pi/4.5)];
[n_x, bin_x] = histc(angle_x, edges);
[n_y, bin_y] = histc(angle_y, edges);
[n_z, bin_z] = histc(angle_z, edges);
%[n, bin] = histc(angle, [min(angle):(max(angle)-min(angle))/10:max(angle)]);
%bar(n);

u_x = pnts2clusters(fr.usage, bin_x);
u_y = pnts2clusters(fr.usage, bin_y);
u_z = pnts2clusters(fr.usage, bin_z);
%u = pnts2clusters(fr.usage, bin);

%% compute clusters of points, which fall in the same bins of the three
%  histograms.
u_xy = [];
i = 1;
while( i <= length(u_x) )

    u_xy = [u_xy cellfun(@(x)(intersect(u_x{i}, x)), u_y, 'UniformOutput', false)];
    
    i = i+1;
    
end

u_xy(cellfun('isempty', u_xy)) = [];

u_xyz = [];
i = 1;
while( i <= length(u_xy) )

    u_xyz = [u_xyz cellfun(@(x)(intersect(u_xy{i}, x)), u_z, 'UniformOutput', false)];
    
    i = i+1;
    
end

u_xyz(cellfun('isempty', u_xyz)) = [];

clusterIndx = u_xyz;
%clusterIndx = u;
clusters = cellfun(@(x)(fr.points3D(1:3, x)), clusterIndx, 'UniformOutput', false);

if(display)
    show_clusters(clusters, 0, [], [], 1);
end
    


