function [clusters, clustersIndx, normclusters] = cluster_through_normals(fr, cam, threshold, display)
% clusters = cluster_through_normals(pnts, normals, threshold, usage, display)
%
% points with similar normals are clustered
% threshold: normals with an Euclidean distance smaller than the threshold value
%            are declared as similar
% usage: specify which points in 'pnts' will be used (optional; default: use
%        all)
% display: 1 - visualize clusters; 0 - no visulaization (optinal; default: 0)

if(nargin < 4)
    display = false;
end

if(~isfield(fr, 'normals'))
    seq.frames = fr;
    seq.camType = cam;
    
    seq = preprocessing(seq, 'normals');
    
    fr = seq.frames;
end

n = fr.normals(:, fr.usage);
pnts = fr.points3D(1:3, fr.usage);
usage = fr.usage;

pos = 1:1:cam.width*cam.height;
pos = pos(usage);

clusters = [];
normclusters = [];
diff = [];
d = [];

% find [0;0;0] normals
posZeroNormals = ismember(n, [0;0;0]);
posZeroNormals = posZeroNormals(1,:) & posZeroNormals(2,:) & posZeroNormals(3,:);
zeroNormals = n(:, posZeroNormals);
pntsZeroNormals = pnts(:, posZeroNormals);

n = n(:, ~posZeroNormals);
pnts = pnts(:, ~posZeroNormals);
pos = pos(:, ~posZeroNormals);

while(~isempty(n))
    
    nCurr = n(:,1);
    pCurr = pnts(:,1);
    iCurr = pos(1);
    
    n(:,1) = [];
    pnts(:, 1) = [];
    pos(1) = [];
    
    % conormality measure:
    % computes angle between two normals n1 and n2
    % angle = acos(n1 * n2) (Scalar Product)
    angles = acos(nCurr'*n);
    indx = find(angles < threshold);
    
    % diff = [n(1,:) - nCurr(1); n(2,:) - nCurr(2); n(3,:) - nCurr(3)];
    % d = sum(diff.^2,1);
    % indx = find(d < threshold);
    
    % cluster points with similar normals
    clusters{length(clusters)+1} = [pCurr pnts(:, indx)];
    clustersIndx{length(clusters)+1} = [iCurr pos(indx)];
    normclusters{length(normclusters)+1} = [nCurr n(:, indx)];
    
    % remove clustered points from the working set
    n(:,indx) = [];
    pnts(:,indx) = [];
    pos(indx) = [];
    
end

normclusters{length(normclusters)+1} = zeroNormals;
clusters{length(clusters)+1} = pntsZeroNormals;

if(display)

    show_clusters(clusters, 100);

end
