function segments = edge_region_growing(bw, display)
% segments = edge_region_growing(bw)
%
% Find all regions in the image "bw" (a black/white image, where edges are shown
% white).
% A region is defined as an area inside of connected edges

if(nargin < 2)
    display = false;
end

scrsz = get(0,'ScreenSize');

height = size(bw, 1);
width = size(bw, 2);
num = height*width;
bwZeros = find(bw == 0);
bwOnes = find(bw == 1);

% -2 denotes edge in image
segments = zeros(height, width);
segments(bwOnes) = -2;

if(display)
    fig = figure('Position', [scrsz(3)/2 scrsz(4)/2 scrsz(3)/2 scrsz(4)/2]); 
    axis off;
    axis equal;
end

c=1;
seed = 0;
posZeros = [];
%while(c<=20)
while(any(any(segments == 0)))
    
    % find a possible seed positions
    posZeros = find(segments == 0);
    ran = random_positions(1, length(posZeros));
    seed = posZeros(ran);
    segments(seed) = -1;
    
    while(any(any(segments == -1)))
        
        p = find(segments == -1);
        p = p(1);
        pc = floor((p - 1) / height) + 1;
        pr = mod(p - 1, height) + 1;
        
        ngbRC = get4ngb(height, width, pr, pc);
        ngb = (ngbRC(2,:)-1)*height + ngbRC(1,:);
        
        n = [];
        
        %for i=1:length(ngb)
        %    if(segments(ngb(i)) == 0)
        %        n = [n ngb(i)];
        %    end
        %end
        n = ngb(find(segments(ngb) == 0));
        
        segments(n) = -1;
        segments(p) = c;
        
    end
    
    if(display)
        figure(fig); 
        imagesc(segments); axis off; axis equal;
        drawnow
    end
        
    c = c+1;
   
end