function [planes, plIndx, patches, rest, restIndx] = pnts2plane_classification_RANSAC(fr, cam, display)
%% [planes, plIndx, patches, rest, restIndx] = pnts2plane_classification_RANSAC(fr, cam, display)
%
% This implementation uses the fact that points determine a bigger plane
% depending on the own normal

if(nargin < 3)
    display = false;
end

if(~isstruct(fr))
    pnts = fr;
    op = generate_oriented_particles(pnts);
    normals = [op.normal];
    deviation = [op.dev];
else

    if(~isfield(fr, 'normals'))
        seq.frames = fr;
        seq.camType = cam;

        seq = preprocessing(seq, 'normals');
        fr = seq.frames;
    end

    pnts = fr.points3D;
    normals = fr.normals;
    deviation = fr.dev;
    usage = fr.usage;
    
end

%% classify if points are planar or not:
thres_dev = mean(deviation) + std(deviation);
planarIndx = find(deviation <= thres_dev); %Swissranger
notPlanarIndx = find(deviation > thres_dev); %Swissranger

planarIndx = planarIndx(ismember(planarIndx, usage));
notPlanarIndx = notPlanarIndx(ismember(notPlanarIndx, usage));
allIndx = planarIndx;

rest = pnts(1:3, notPlanarIndx);
restIndx = notPlanarIndx;

%% RANSAC-parameters:

maxIter = 40;
minIn = 30;
inlierIndx = [];
bench_ang = [];
bench_dis = [];
n = [];

planes = [];
plIndx = [];

ln = length(allIndx);

%% main loop:

while(1)
 
    counter = 0;
    n = size(allIndx,2);
    tIn = 0.9*n;
    inlierIndx = [];
    bench_ang = [];
    bench_dis = [];
    
    while(1)

        if(counter > maxIter) break; end

        % select randomly one point with normal
        pos = random_positions(1, n);

        currNormal = normals(:, allIndx(pos));
        currPnt = pnts(1:3,allIndx(pos));
        currD = currNormal'*currPnt;

        angles = acos(currNormal'*normals(:, allIndx));
        f = find(angles > pi/2);
        angles(f) = pi - angles(f);

        in1 = find(angles < (4/18 * pi));
        
        dis = abs(currNormal'*[pnts(1:3,allIndx)] - currD);
        in2 = find(dis < 100);      
        
        in = intersect(in1, in2);

        if(length(in) < minIn)
            counter = counter+1;
            continue;
        else

            [coeff, score, roots] = princomp(pnts(1:3,allIndx(in))');

            if(length(in) > tIn)
                inlierIndx = allIndx(in);
                bench_dis = sum(abs(score(3,:)));
                bench_ang = sum(angles(in));
                break;
            end

            if(length(in) == length(inlierIndx))

                if(sum(abs(score(3,:))) < bench_dis & sum(angles(in)) < bench_ang)

                    inlierIndx = allIndx(in);
                    bench_dis = sum(abs(score(3,:)));
                    bench_ang = sum(angles(in));

                end

            elseif(length(in) > length(inlierIndx))

                inlierIndx = allIndx(in);
                bench_dis = sum(abs(score(3,:)));
                bench_ang = sum(angles(in));
                
            else

            end

            counter = counter+1;

        end

    end

    %disp(['counter: ' num2str(counter) '; size: ' num2str(length(inlierIndx))]);
    [i, ia, ib] = intersect(allIndx, inlierIndx);
    allIndx(ia) = [];
    %disp(['rest: ' num2str(length(allIndx))]);

    currPlanes = {pnts(1:3,inlierIndx)};
    
    planes = [planes currPlanes];
    plIndx = [plIndx {inlierIndx}];
    
    if(length(allIndx) == ln | isempty(allIndx)) 
        break; 
    else
        ln = length(allIndx);
    end
    
    %j = j+1;
    
end


restIndx = [restIndx allIndx];
rest = pnts(1:3,restIndx);

[patches, r, rIndx] = cluster2plane(planes, plIndx);
rest = [rest r];
restIndx = [restIndx rIndx];

if(display)
    fig = show_clusters({patches.pnts}, 0, []);
    fig = plot_3Dpoints(rest, 'k', fig, 4); 
end

