function [planes, rest, patchesIndx, restIndx] = pnts2plane_classification_rgrowpure(fr, cam, display)
%% [planes, rest, patchesIndx, restIndx] = pnts2plane_classification_rgrow(fr, cam, display)
%
%   This function is an implementation of the algorithm described Olaf
%   Kaehler in "On Fusion of Range and Intensity Information Using
%   Graph-Cut for Planar Patch Segmentation

%% some default values:

if(nargin < 3)
    display = false;
end

if(~isfield(fr, 'indx8'))
    seq.frames = fr;
    seq.camType = cam;
    
    seq = preprocessing(seq, 'ngb');
    fr = seq.frames;
end

rest = [];
restIndx = [];

usage = fr.usage;
pnts = fr.points3D(1:3,:);
allIndx = usage;
threshold = 30;


%% start region growing:

patchesIndx = [];
patches = [];
i = 1;
while(~isempty(allIndx))
    
    % initialize seed:
    initSeedIndx = [allIndx(1) fr.indx8{allIndx(1)}];
    initSeedIndx = intersect(initSeedIndx, allIndx);
    
    [p, n, d] = main_plane_extraction(pnts(:,initSeedIndx));
    
    if(isempty(p)) 
        
        if(length(allIndx) <= 3)
            restIndx = [restIndx allIndx];
            allIndx = [];
        else
            restIndx = [restIndx allIndx(1)];
            allIndx(1) = [];
        end
        
        continue; 
    
    end
      
    workingIndx =  initSeedIndx;
    
    allIndx(ismember(allIndx, workingIndx)) = [];
    
    patchesIndx{i} = [];
    
    while(~isempty(workingIndx))
        
        currIndx = workingIndx(1);
        workingIndx(1) = [];
        patchesIndx{i} = [patchesIndx{i} currIndx];
        
        [n, d] = plane_computation(pnts(:,[patchesIndx{i} workingIndx]));
        
        % find indices of neighbors
        ngbIndx = fr.indx8{currIndx};
        
        % find the indices, that are unprocessed until now
        ngbIndx = intersect(ngbIndx, allIndx);
        
        if(isempty(ngbIndx)) continue; end
        
        % criterion for assignment to current plane:
        decider = n'*pnts(:,ngbIndx) - d;
        in = find(decider < threshold);
        
        ngbIndx = ngbIndx(in);
        %allIndx(ismember(allIndx, ngbIndx)) = [];
        
        workingIndx = [workingIndx ngbIndx];
        workingIndx = unique(workingIndx);
        workingIndx(ismember(workingIndx, patchesIndx{i})) = [];
        
    end
    
    patches{i} = pnts(1:3, patchesIndx{i});
    allIndx(ismember(allIndx, patchesIndx{i})) = [];
    
    i = i+1;
    
end
    
%% display results:

if(display)
    fig = show_clusters(patches, 50);
    fig = plot_3Dpoints(rest, 'k', fig);
end

planes = patches;
rest = pnts(:, restIndx);
