function frame = svd_enhancing(fr, camType)
%% frame = svd_enhancing(fr, camType)
%
% This function is based on the paper:
% "SwissRanger SR-3000 Range Images Enhancement by a Singular Value
% Decomposition Filter"
% from
% GuruPrasad M. Hedge and Cang Ye
%
% problem: normals needed to be computed => time consuming

%% build NERI image (Normal-Enhanced Range Image)

neri = zeros(camType.height, camType.width, 3);
neri(:,:,1) = reshape(fr.normals(1,:), camType.width, [])';
neri(:,:,2) = reshape(fr.normals(2,:), camType.width, [])';
neri(:,:,3) = reshape(fr.distances, camType.width, [])';

neriG = mean(neri, 3);

%% decompose NERI image and determine appropiate threshold
[U,S,V] = svd(neriG);

Sdach = diag(S)./max(diag(S));
gdach = abs(gradient(Sdach));
pos = find(gdach < 0.001);

S(pos(1):end,pos(1):end) = 0;

%% compose new image
disNew = U*S*V';

frame = fr;
frame.distances = reshape(disNew', 1, []);
frame = backprojection_fast(frame, camType);
