function feat = compute_feature(planes, method)
%% feat = compute_feature(planes, method)

%% histopt determines whether a data point contributes to its neighboring
% bins or not
histopt = 'hard';
% histopt = 'smooth';

feat = [];

if(length(planes) == 1)
    return;
end

comb = nchoosek(1:length(planes), 2);
comb = mat2cell(comb, ones(1,size(comb,1)), 2);

%% angles between all planes
if(strcmp(method, 'angles') | strcmp(method, 'all'))
    
    angles = cell2mat(cellfun(@(x)(get_angle(planes(x(1)).n, planes(x(2)).n)), comb, 'UniformOutput', false))';    
    
    if(strcmp(histopt, 'hard'))
        
        anghist = histc(angles, [0:pi/18:pi/2]);
        anghist = anghist(1:end-1)./sum(anghist);
        
    end
    
    if(strcmp(histopt, 'smooth'))
        
        means = pi/36:pi/18:pi/2;
        var = pi/18;
        
        anghist = cell2mat(arrayfun(@(x)(sum(exp(-0.5 *((angles-x)/var).^2))), means, 'UniformOutput', false));
        anghist = anghist./sum(anghist);
        
    end
        
    feat = [feat; anghist'];
    
end

%% shapes of the planes defined by ratio of the main axis
if(strcmp(method, 'shapes') | strcmp(method, 'all'))
    
    shapes = [planes.shape];
    shaphist = [];
    
    if(strcmp(histopt, 'hard'))
        
        shaphist = histc(shapes, [0:0.2:1]);
        shaphist = shaphist(1:end-1)./sum(shaphist);
        
    end
    
    if(strcmp(histopt, 'smooth'))
        
        means = 0.1:0.2:1;
        var = 0.2;
        
        shaphist = cell2mat(arrayfun(@(x)(sum(exp(-0.5 *((shapes-x)/var).^2))), means, 'UniformOutput', false));
        shaphist = shaphist./sum(shaphist);
        
    end
    
    feat = [feat; shaphist'];
    
end

%% ratio of the areas
if(strcmp(method, 'arearatio') | strcmp(method, 'all'))
    
    arearatio = cell2mat(cellfun(@(x)(min(planes(x(1)).area, planes(x(2)).area)/max(planes(x(1)).area, planes(x(2)).area)), comb, ...
         'UniformOutput', false))';
    
    arhist = zeros(1,length([0:0.2:1])-1);
    
    if(strcmp(histopt, 'hard'))
        
        arhist = histc(arearatio, [0:0.2:1]);
        arhist = arhist(1:end-1)./sum(arhist);
        
    end
    
    if(strcmp(histopt, 'smooth'))
        
        means = 0.1:0.2:1;
        var = 0.2;
        
        arhist = cell2mat(arrayfun(@(x)(sum(exp(-0.5 *((arearatio-x)/var).^2))), means, 'UniformOutput', false));
        arhist = arhist./sum(arhist);
        
    end
    
    feat = [feat; arhist'];
    
end

if(strcmp(method, 'areas') | strcmp(method, 'all'))
    
    areas = [planes.area];
    
    ahist = histc(areas, [0 (([0.25 0.5 1:3])*1000).^2 realmax]);
    ahist = ahist(1:end-1)./sum(ahist);
    
    feat = [feat; ahist'];
    
end

