function fig = plot_3Dpoints(pnts, color, fig, msize, marker, values)
%% fig = plot_3Dpoints(pnts, color, fig, msize)
%         -              -
%        |p11 p21 p31     |
% pnts = |p12 p22 p32 ... |
%        |p13 p23 p33     |
%         -              -
%
% color: 'r', 'g', 'b', 'c', 'm', 'y', 'k', 'w', 'jet', 'copper', 'bone'
%
% fig: handle for a specific figure
%
% msize: size of the marker of the 3D points

if(nargin < 2)
    color = 'jet';
end

if(nargin < 3 || isempty(fig))
    fig = figure; axis equal;
    set(fig, 'Name', '3D point cloud');
else
    fig = figure(fig);
    axis equal;
end

if(nargin < 4 || isempty(msize))
    msize = 4;
end

if(nargin < 5)
    marker = '.';
end

if(nargin < 6)
    values = pnts(3,:);
end

hold on;
axis off;
axis equal;

if(isempty(pnts)) fig = figure(fig); return; end

X = pnts(1,:);
Y = pnts(2,:);
Z = -1*pnts(3,:);
mi = min(values);
ma = max(values);

if(strcmp(color, 'jet') | strcmp(color, 'copper') | strcmp(color, 'bone'))
    
    % whitebg('w');
    
    %mi = 0;
    %ma = 7500;
    
    map = colormap(color);
    if(strcmp(color, 'bone'))
        map = map(end:-1:1,:);
    end
    
    steps = length(map)-1;
    range = (ma - mi);
    
    p = [];
    i = 1;
    tdown = mi;
    tup = mi + i/steps * range;
    while(i <= steps)
        
        
        p = find(values >= tdown & values <= tup);
        plot3(X(p), Y(p), Z(p), marker, 'Color', map(i, :), 'MarkerSize', msize);
        
        i = i+1;
        tdown = tup;
        tup = mi + i/steps * range;
        
    end
    
    p = find(values > tup);
    plot3(X(p), Y(p), Z(p), marker, 'Color', map(end, :), 'MarkerSize', msize);

elseif(~isstr(color))
    
    % whitebg('w');
    if(size(color,1) > 1)
        
        i = 1;
        while(i <= size(pnts,2))
            plot3(X(i),Y(i),Z(i), marker, 'Color', color(i,:), 'MarkerSize', msize);
            i = i+1;
        end
    
    else
        plot3(X,Y,Z, marker, 'Color', color, 'MarkerSize', msize);
    end
    
else

    % whitebg('w');

    plot3(X,Y,Z, [marker color], 'MarkerSize', msize);

end

figure(fig);
hold off;
