function [fig, map] = show_clusters(clusters, minSize, fig, method, s, p, marker, cMap)
% fig = show_clusters(clusters, minSize, fig, method, s, p, marker, cMap)
% 
% methods: -'points'; 'convexhull'

axis off; axis equal;

if(nargin < 7)
    marker = '.';
end

if(nargin < 6)
    p = false;
end

if(nargin < 5)
    s = [];
end

if(nargin < 4 || isempty(method))
    method = 'points';
end

if(nargin < 3 || isempty(fig))
    %fig = figure('Position', [1 1 600 600]);
    fig = figure;
else
    fig = figure(fig);
end

if(nargin < 2)
    minSize = 0;
end

if(isempty(minSize))
    minSize = 0;
end

if(strcmp(method, 'convexhull') & isempty(s))
    s = input('Specific color for all hulls? \n [default: press enter - each hull gets an other color]\n');
end

rest = [];
in = clusters;

l = cellfun('size', clusters, 2);
%[ls, li] = sort(l, 'descend');
%in = in(li);
ls = l;

lo = find(ls < minSize);

if(~isempty(lo))
    rest = cat(2, clusters{lo});
end

in(lo) = [];

if(nargin > 7)
    map = cMap;
else
    map = generate_colormap(length(in));
    map = abs(map);
    map(find(map > 1)) = 1;
end

if(p)
    perm = randperm(length(in));
    in = in(perm);
end

i = 1;
while(i <= length(in))

    if(strcmp(method, 'points'))
        fig = plot_3Dpoints(in{i}, map(i,:), fig, 6, marker);
    elseif(strcmp(method, 'convexhull'))
        hold on;
        X = in{i}';
        K = convhulln(X(:,1:3));
        
        if(isempty(s))
            trisurf(K,X(:,1),X(:,2), -X(:,3), repmat([i], 1, size(K,1)), 'FaceAlpha', 0.5);
        else
            trisurf(K,X(:,1),X(:,2), -X(:,3), 'FaceAlpha', 0.5, 'FaceColor', map(i,:));
        end
        
    else
        disp('Error: unknown display method!');
    end
        
    i = i+1;

end

if(p)
    map = map(perm,:);
end

if(~isempty(rest))
    fig = plot_3Dpoints(rest, 'k', fig);
end